﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IdentityModel.Tokens;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace Microsoft.GGTool.Common
{
	public static class BootstrapTokenTools
    {
		public static string ClaimActive = "http://schemas.microsoft.com/cgg/2010/identity/claims/profilestate";

		public static bool IsActive(string token)
        {
            try
            {                
                return GetValueByClaimType(ClaimActive, GetAttrStatement(token)) == "Active";
            }
            catch (Exception ex)
            {
                Logger.WriteLog(ex);
            }
            return false;
        }

        public static string GetValueByClaimType(string claimType, SamlAttributeStatement tokenAttributeStatement)
        {
            var valueAttr = tokenAttributeStatement.Attributes.Single(a => a.Namespace + "/" + a.Name == claimType).AttributeValues.First();
            return valueAttr;
        }


        private static SamlAttributeStatement GetAttrStatement(string token)
        {
            SamlSecurityToken stoken;
            using (StringReader sr = new StringReader(token))
            {
                using (XmlReader reader = XmlReader.Create(sr))
                {
                    if (!reader.ReadToFollowing("saml:Assertion"))
                    {
                        throw new Exception("Assertion not found!");
                    }
                    SecurityTokenHandlerCollection collection = SecurityTokenHandlerCollection.CreateDefaultSecurityTokenHandlerCollection();
                    stoken = (SamlSecurityToken)collection.ReadToken(reader.ReadSubtree());
                }
            }
            return (SamlAttributeStatement)stoken.Assertion.Statements.First();
        }

    }
}
