﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Newtonsoft.Json.Linq;

namespace Microsoft.GGTool.Common
{
    public static class GGTool
    {
        public static string GetActAsToken(string bootstrapToken, string audience, string federationActAsEndpoint)
        {
            try
            {

                var dic = new System.Collections.Specialized.NameValueCollection();
                dic.Add("grant_type", "urn:ietf:params:oauth:grant-type:token-exchange");
                dic.Add("audience", audience);
                dic.Add("subject_token", bootstrapToken);
                dic.Add("subject_token_type", "urn:ietf:params:oauth:token-type:jwt");

                string actAsToken = null;
                using (WebClient client = new WebClient())
                {
                    client.Headers[System.Net.HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
                    var response = client.UploadValues(federationActAsEndpoint, dic);
                    var jObject = JObject.Parse(System.Text.Encoding.UTF8.GetString(response));
                    actAsToken = jObject["access_token"].ToString();
                }

                return actAsToken;
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public static bool ValidateActAsToken(string actAsToken)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(actAsToken))
                    return false;

                var parts = actAsToken.Split('.');

                string partToConvert = parts[1];
                partToConvert = partToConvert.Replace('-', '+');
                partToConvert = partToConvert.Replace('_', '/');
                switch (partToConvert.Length % 4)
                {
                    case 0:
                        break;
                    case 2:
                        partToConvert += "==";
                        break;
                    case 3:
                        partToConvert += "=";
                        break;
                }

                var partAsBytes = Convert.FromBase64String(partToConvert);
                var partAsUTF8String = Encoding.UTF8.GetString(partAsBytes, 0, partAsBytes.Count());

                var jwtToken = JObject.Parse(partAsUTF8String);

                var exp = jwtToken["exp"].Value<int>();

                var expDateTime = new DateTime(1970, 1, 1, 0, 0, 0).AddSeconds(exp);

                return DateTime.UtcNow < expDateTime;

            }
            catch (Exception e)
            {
                Logger.WriteLog(e);
            }
            return false;
        }

        public static XmlDocument SendMessage(string uri, string token, string jsonMsg)
        {
            var client = new HttpClient();

			client.DefaultRequestHeaders.Add("Authorization", token);
            var content = new StringContent(jsonMsg, Encoding.UTF8, "application/json");

            var result = client.PostAsync(uri, content).Result;
            string jsonRes = result.Content.ReadAsStringAsync().Result;
            var xmlResult = GGConvert.ToXml(jsonRes);
            return xmlResult;
        }


        public static XmlNamespaceManager GetXmlnsManager(XmlNameTable xmlNameTable)
        {
            XmlNamespaceManager xmlnsManager = new XmlNamespaceManager(xmlNameTable);
            xmlnsManager.AddNamespace("gov", "http://www.govtalk.gov.uk/CM/envelope");

            return xmlnsManager;
        }

        public static string GetCorrelationId(XmlDocument ggResponse)
        {
            var res = ggResponse.DocumentElement.SelectSingleNode("//gov:Header/gov:MessageDetails/gov:CorrelationID", GetXmlnsManager(ggResponse.NameTable))?.InnerText;
            return res;
        }

        public static string GetClass(XmlDocument ggResponse)
        {
            var res = ggResponse.DocumentElement.SelectSingleNode("//gov:Header/gov:MessageDetails/gov:Class", GetXmlnsManager(ggResponse.NameTable))?.InnerText;
            return res;
        }

        public static string GetQualifier(XmlDocument ggResponse)
        {
            var res = ggResponse.DocumentElement.SelectSingleNode("//gov:Header/gov:MessageDetails/gov:Qualifier", GetXmlnsManager(ggResponse.NameTable))?.InnerText;
            return res;
        }

    }
}
